import json
import unittest

from .. import zanza

class ZanzaTests(unittest.TestCase):

    def test_empty_string(self):
        """Zanza with empty string.
        """

        with self.assertRaises(ValueError):
            zanza("")

    def test_ok(self):
        """Zanza with valid input.
        """

        res = zanza("foobarbaz")
        expected = [[1, 0, 2], 9, 0, -13, -1, 17, -16, -1, 25]
        self.assertEqual(res, expected)



import unittest

from .. import dezanza

class DezanzaTests(unittest.TestCase):

    def test_empty_seq(self):
        """Dezanza with empty string.
        """

        with self.assertRaises(ValueError):
            dezanza([])

    def test_invalid_format(self):
        """Dezanza with invalid input format.
        """

        with self.assertRaises(ValueError):
            dezanza([102, 3, 4, 5])

    def test_ok(self):
        """Dezanza with valid input.
        """

        res = dezanza([[102], 9, 0, -13, -1, 17])
        expected = "foobar"
        self.assertEqual(res, expected)



#!/usr/bin/env python

import argparse
import json
import logging

from . import dezanza, __doc__ as manual

logger = logging.getLogger(__name__)

def main():
    """Deobfuscator main program.
    """

    parser = argparse.ArgumentParser(
        description="Deobfuscate the given input sequence",
        epilog=manual,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("source", help="JSON-encoded input sequence to deobfuscate")
    args = parser.parse_args()

    try:
        zj = json.loads(args.source)
        zs = dezanza(zj)
        print(zs)
    except Exception as ex:
        logger.error("ERROR: %s" % ex)


if __name__ == "__main__":
    main()



"""
This script obfuscates the input string by the following algorithm:

- Takes the first character of the input string, and gets its Unicode
  code point (character code)
- Sets this char code as a reference value
- Initializes a *list* in which stores the digits of the previous
  value as a *list*
- Continues to the next character by calculating its code point and
  the delta to the previous character's code point.
- Sets the current code point as the next reference value
- Adds this delta value to the return list and repeats the steps
  above to the input strings length.
"""

def zanza(source):
    """Obfuscate the input string.

    Args:
        source (str): Input string
    """

    if len(source) == 0:
        raise ValueError("Invalid input: zero-length string")

    ret = []
    prev = ord(source[0])
    first = map(int, str(prev))
    ret.append(list(first))

    for s in source[1:]:
        curr = ord(s)
        ret.append(curr - prev)
        prev = curr

    return ret



"""
This script deobfuscates the input sequence that was
obfuscated by `zanza`.

The input needs to be a JSON-encoded list passed as a
string. To not to confuse the shell with special
characters like [ and ], the input has to be wrapped
in double quotes.
"""

def _reduce(fn, iterable, initializer):
    """Helper method to reduce the input list
    to a single value.

    Args:
        fn: Callback function
        iterable: Input list
        initializer: Initial value
    """

    it = enumerate(iterable)
    value = initializer
    for idx, element in it:
        value = fn(value, element, idx)
    return value


def dezanza(source):
    """Deobfuscate the input sequence.

    Args:
        source (list): JSON-encoded list
    """

    if len(source) == 0:
        raise ValueError("Invalid input: zero length sequence")

    first = source[0]

    if type(first) is not list:
        raise ValueError("Invalid input: first element needs to be a list")

    ret = ""
    first.reverse()
    prev = _reduce(lambda acc, val, idx: acc + val * 10 ** idx, first, 0)
    ret += chr(prev)

    for item in source[1:]:
        curr = prev + item
        ret += chr(curr)
        prev = curr

    return ret



#!/usr/bin/env python

import argparse
import json
import logging

from . import zanza, __doc__ as manual

logger = logging.getLogger(__name__)

def main():
    """Obfuscator main program.
    """

    parser = argparse.ArgumentParser(
        description="Obfuscate the given input string",
        epilog=manual,
        formatter_class=argparse.RawDescriptionHelpFormatter)
    parser.add_argument("source", help="Input string to obfuscate")
    args = parser.parse_args()

    try:
        z = zanza(args.source)
        zj = json.dumps(z)
        print(zj)
    except Exception as ex:
        logger.error("ERROR: %s" % ex)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

from setuptools import find_packages
from distutils.core import setup

entry_points = {
    "console_scripts": [
        "zanza = zanza.__main__:main",
        "dezanza = dezanza.__main__:main"
    ]
}

setup(
    name="zanza",
    version="0.1.0",
    author="fogasl",
    description="Dead-simple string obfuscation library",
    long_description=open("README.md", "r", encoding="utf8").read(),
    long_description_content_type="text/markdown",
    keywords="simple string obfuscation",
    license="BSD-3-Clause",
    python_requires=">3",
    packages=find_packages(),
    entry_points=entry_points
)
